/**
 * File:
 *  fader.c
 *
 * Notes:
 *  This file contains the functions for my own Fader routines.
 *
 * Functions:
 *  initFader          Initialises a Fader and adds it to the internal linked list
 *  serviceFaders      Services all initialised Faders
 *
 * History:
 *  09/09/2012 Written by Simon Platten
 */
#include "fader.h"
/**
 * Initialise fader
 *
 * Parameters:
 *  pF, pointer to an instance of a fader node
 *  pPort, the I/O port containing the LED(s)
 *  bytIOmask, the I/O mask containing the LED(s) to use
 *  eMode, either SINGLE_FADER or MULTIPLE_FADERS
 *  eDir, either FADE_IN or FADE_OUT
 *  blnLoop, TRUE or FALSE
 *  blnBounce, TRUE or FALSE
 *  blnEnable, TRUE or FALSE
 *  uintDelta, rate of change delta / speed, 1 = Slowest
 *  uintRepeats, the number of repeats per fade iteration
 *
 * Returns:
 *  TRUE if ok, FALSE if error
 */
boolean initFader(faderDef        *pF,
                  ushort          *pPort,
                  byte            bytIOmask,
                  eFadeMode       eMode,
                  eFadeDirections eDir,
                  boolean         blnLoop,
                  boolean         blnBounce,
                  boolean         blnEnable,
                  byte            bytDelta,
                  uint            uintRepeats) {
// Validate parameters
  if ( !(pF
    &&  pPort
    &&  bytIOmask > 0
    && (eMode == SINGLE_FADER || eMode == MULTIPLE_FADERS)) ) {
// Invalid parameters do nothing
    return FALSE;
  }
// Delta cannot be 0
  if ( bytDelta == 0 ) {
    bytDelta = 1;
  }
// Set-up the structure
  pF->uBounce     = (blnBounce == TRUE) ? 1 : 0;
  pF->uDir        = (eDir == FADE_OUT)  ? 1 : 0;
  pF->uEnable     = (blnEnable == TRUE) ? 1 : 0;
  pF->uLoop       = (blnLoop == TRUE)   ? 1 : 0;
  pF->uMode       = (eMode == MULTIPLE_FADERS) ? 1 : 0;
  pF->uMode       = (eMode == MULTIPLE_FADERS) ? 1 : 0;
  pF->uState      = 0;
  pF->bytBounces  = 0;
  pF->pPort       = pPort;
  pF->bytIOmask   = bytIOmask;
  pF->bytDelta    = bytDelta;
  pF->uintRepeats = uintRepeats;
  pF->uintCount   = pF->uintRptCtr = 0;

  if ( pF->uDir == FADE_IN ) {
    pF->uintDutyTime = 1;
    pF->intOffTime = FADER_START_PT;
  } else {
    pF->uintDutyTime = FADER_START_PT;
    pF->intOffTime = 1;
  }
  return TRUE;
}
/**
 * Services and updates faders
 */
/*
void serviceFaders(void) {
  faderDef* pF;
  for( pF=pFLL; pF!=NULL; pF=pF->pNext ) {
    if ( pF->uEnable == 0 ) {
// Skip this fader it isn't enabled
      continue;
    }
    if ( pF->uintCount-- == 0 ) {
      if ( pF->uState == 1 && pF->intOffTime > 0 ) {
        pF->uintCount = (uint)pF->intOffTime;
        pF->uState = 0;
      } else {
        pF->uintCount = pF->uintDutyTime;
        pF->uState = 1;
      }
      if ( pF->uintRptCtr++ >= pF->uintRepeats ) {
        if ( pF->uDir == FADE_IN && pF->intOffTime > 0 ) {
          if ( pF->intOffTime - (int)pF->bytDelta > 0 ) {
            pF->intOffTime = pF->intOffTime - (int)pF->bytDelta;
          } else {
            pF->intOffTime = 0;
          }
        } else if ( pF->uDir == FADE_OUT && pF->intOffTime <= FADER_START_PT ) {
          if ( pF->intOffTime + (int)pF->bytDelta > FADER_START_PT ) {
            pF->intOffTime = FADER_START_PT;
          } else {
            pF->intOffTime = pF->intOffTime + (int)pF->bytDelta;
          }
        }
        pF->uintDutyTime = FADER_START_PT - pF->intOffTime;
        pF->uintRptCtr = 0;

        if ( pF->uDir == FADE_IN && pF->intOffTime == 0 ) {
          if ( pF->uLoop ) {
            pF->uintDutyTime = 1;
            pF->intOffTime = FADER_START_PT;
            pF->uintCount = pF->uintDutyTime;
          }
          if ( pF->uBounce ) {
            pF->uDir = FADE_OUT;
            pF->uintDutyTime = FADER_START_PT;
            pF->uintCount = pF->uintDutyTime;
            pF->intOffTime = 1;
            pF->bytBounces++;
          }
          if ( !pF->uLoop && (!pF->uBounce
                           || (pF->uBounce && pF->bytBounces > 1)) ) {
            pF->uEnable = 0;
          } else {
            pF->uEnable = 1;
          }
          pF->uState = 1;
        } else if ( pF->uDir == FADE_OUT && pF->uintDutyTime == 0 ) {
          if ( pF->uLoop ) {
            pF->uintDutyTime = FADER_START_PT;
            pF->uintCount = pF->uintDutyTime;
            pF->intOffTime = 1;
          }
          if ( pF->uBounce ) {
            pF->uDir = FADE_IN;
            pF->uintDutyTime = 1;
            pF->intOffTime = FADER_START_PT;
            pF->uintCount = pF->uintDutyTime;
            pF->bytBounces++;
          }
          if ( !pF->uLoop && (!pF->uBounce
                           || (pF->uBounce && pF->bytBounces > 1)) ) {
            pF->uEnable = 0;
          } else {
            pF->uEnable = 1;
          }
          pF->uState = 0;
        }
      }
    }
    if ( pF->uState ) {
// Turn On
      if ( pF->uMode == MULTIPLE_FADERS ) {
        *pF->pPort |= pF->bytPin;
      } else {
        *pF->pPort |= (1 << pF->bytPin);
      }
    } else {
// Turn Off
      if ( pF->uMode == MULTIPLE_FADERS ) {
        *pF->pPort &= 0xff ^ pF->bytPin;
      } else {
        *pF->pPort &= 0xff ^ (1 << pF->bytPin);
      }
    }
  }
}
*/